#!/bin/ksh

# Copyright 2006-2011 Carl Thijssen

# This file is part of the Ultimate Deployment Appliance.
#
# Ultimate Deployment Appliance is free software; you can redistribute
# it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# Ultimate Deployment Appliance is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

PATCHDIR=$1

VERSION=2.0
BUILD=23
LOGFILE=$VERSION\_$BUILD.$$.log

PUBLIC=/var/public
LOGDIR=/tmp
BINDIR=$PUBLIC/bin
CGIBINDIR=$PUBLIC/cgi-bin
CGIBINOSDIR=$PUBLIC/cgi-bin/os
CGIBINSVCDIR=$PUBLIC/cgi-bin/services
WWWDIR=$PUBLIC/www
WWWSUPDIR=$PUBLIC/www/supportmatrix
WWWSUPVMDIR=$PUBLIC/www/supportmatrix/vmware
ICONS=$WWWDIR/icon
JSDIR=$WWWDIR/js
TEMPLATES=$WWWDIR/templates
CONFDIR=$PUBLIC/conf
CONFMOUNTSDIR=$PUBLIC/conf/mounts
VERSIONDIR=$CONFDIR/version
TFTPROOT=$PUBLIC/tftproot
FILESDIR=$PUBLIC/files
PATCHTOPDIR=$PUBLIC/patches

if [ "$PATCHDIR" == "" ]
then
  PATCHDIR=$PATCHTOPDIR/patch.$VERSION.$BUILD.$$
else
  cd $PATCHDIR
fi

log()
{
  TIMESTAMP=`date +%Y%m%d-%H%M%S`
  echo $*
  echo $TIMESTAMP $* >> $LOGDIR/$LOGFILE
}


################ copy actions #########################
copy_tftproot_files()
{
  ls -1 files/tftproot | while read i
   do
     copy_file files/tftproot/$i $TFTPROOT
   done
}

copy_bin_files()
{
  ls -1 files/bin | while read i
   do
     copy_file files/bin/$i $BINDIR
   done
}

copy_cgi_bin_files()
{
  ls -1 files/cgi-bin | while read i
  do
     copy_file files/cgi-bin/$i $CGIBINDIR
  done
}

copy_cgi_bin_os_files()
{
  ls -1 files/cgi-bin/os | while read i
  do
     copy_file files/cgi-bin/os/$i $CGIBINOSDIR
  done
}

copy_conf_files()
{
  ls -1 files/conf | while read i
  do
     if [ -f files/conf/$i ]
     then
       copy_file files/conf/$i $CONFDIR
     fi
  done
}

copy_www_files()
{
  ls -1 files/www | while read i
  do
     if [ -f files/www/$i ]
     then
       copy_file files/www/$i $WWWDIR
     fi
  done
}

copy_www_support_files()
{
  ls -1 files/www/supportmatrix | while read i
  do
     if [ -f files/www/supportmatrix/$i ]
     then
       copy_file files/www/supportmatrix/$i $WWWSUPDIR
     fi
  done
}

copy_www_support_vmware_files()
{
  ls -1 files/www/supportmatrix/vmware | while read i
  do
     if [ -f files/www/supportmatrix/vmware/$i ]
     then
       copy_file files/www/supportmatrix/vmware/$i $WWWSUPVMDIR
     fi
  done
}

copy_template_files()
{
  ls -1 files/www/templates | while read i
  do
     if [ -f files/www/templates/$i ]
     then
       copy_file files/www/templates/$i $TEMPLATES
     fi
  done
}

copy_js_files()
{
  ls -1 files/www/js | while read i
  do
     if [ -f files/www/js/$i ]
     then
       copy_file files/www/js/$i $JSDIR
     fi
  done
}


copy_file()
{
  FILE=$1
  DSTDIR=$2
  BASEFILE=`basename $FILE`
  log Copying file $FILE to $DSTDIR
  echo -n "  $FILE in $DSTDIR"
  diff $FILE $DSTDIR/$FILE > /dev/null 2>&1
  if [ $? -eq 0 ]
  then
    log Keeping Current version
    echo " OK! (current version of $FILE is ok)"
  else 
    log Backing up file to $PATCHDIR/before/$BASEFILE
    cp -f -p $DSTDIR/$FILE $PATCHDIR/before/$BASEFILE
    log Replacing $DSTDIR/$BASEFILE for new version $FILE
    cp -f -p $FILE $DSTDIR/$BASEFILE
    if [ $? -ne 0 ]
    then
      log ERROR: Replace Failed
      if [ ! -d $DSTDIR ]
      then
        log ERROR: $DSTDIR does not exist
        echo "FAILED! (Destination dir $DSTDIR does not exist)"
      else
        if [ ! -f $FILE ]
        then
          log ERROR: Source file $FILE does not exist
          echo "FAILED! (Source file $FILE does not exist)"
        else
          log ERROR: Unknown error
          echo "FAILED!"
        fi
      fi
    else
      log OK: Copied $FILE to $DSTDIR/$BASEFILE
      cp -f -p $DSTDIR/$FILE $PATCHDIR/after/$BASEFILE
      log Copied $FILE to $PATCHDIR/after/$BASEFILE
    fi
  fi
}

overwrite_conf_files()
{
  log Overwrite conf files
  for file in \
      os.conf \
      nfsexportheader.conf
  do
     if [ -f $CONFDIR/$file ]
     then
      BACKUP=$file.$VERSION.$BUILD.backup.$$
      log Found current file $file, backing it up
      cp -f -p $CONFDIR/$file $CONFDIR/$BACKUP
      cp -f -p $CONFDIR/$file $PATCHDIR/before/$BACKUP
     fi
     log Overwriting $file
     cp -f -p $CONFDIR/$file.overwrite $CONFDIR/$file
     if [ $? -eq 0 ]
     then
       log OK replaced $file
       cp -f -p $CONFDIR/$file $PATCHDIR/after/$BACKUP
       if [ $? -eq 0 ] 
       then
         log Removing .overwrite file
         rm $CONFDIR/$file.overwrite
       fi
     else
       log FAILED to replace file $CONFDIR/$file with $CONFDIR/$file.overwrite
     fi
  done
}


create_file_links()
{
  log Creating File links

  ln -sf /var/public/files/smb          /etc/init.d/smb
  ln -sf /var/public/files/syslog.conf  /etc/syslog.conf

  rm -f /etc/syslog.conf.org
  rm -f /etc/init.d/smb.org

}


change_file_permissions()
{
  log Changing permissions on public directory
  chmod -R 755 /var/public/bin
  chown -hR apache.apache /var/public/bin

  chmod -R 755 /var/public/cgi-bin
  chown -hR apache.apache /var/public/cgi-bin

  chmod -R 755 /var/public/www/supportmatrix
  chown -hR apache.apache /var/public/www/supportmatrix

  chmod -R 755 /var/public/www/templates
  chown -hR apache.apache /var/public/www/templates

  chmod -R 755 /var/public/www/js
  chown -hR apache.apache /var/public/www/js

  chmod 755 /var/public/www/*
  chown -h apache.apache /var/public/www/*

  chmod -R 755 /var/public/conf
  chown -hR apache.apache /var/public/conf

  chmod -R 755 /var/public/files
  chown -hR apache.apache /var/public/files

  chmod 755 /var/public/tftproot/*
  chown -h apache.apache /var/public/tftproot/*

}

############################   Main routines ########################
prepare_upgrade()
{
  log Preparing upgrade...

  mkdir -p $PATCHDIR
  if [ $? -eq 0 ]
  then
    log OK: Created patchdir $PATCHDIR
    cd $PATCHDIR
  else
    log ERROR: Could not create patchdir $PATCHDIR
    exit 2
  fi

  for newdir in $PATCHDIR/before $PATCHDIR/after $PATCHDIR/version
  do
    log Creating directory $newdir
    mkdir $newdir
    if [ $? -eq 0 ]
    then
      log OK: Created directory $newdir
    else
      log ERROR: Could not create $newdir
      exit 1
    fi
  done

}

determine_current_versions()
{
  log Determining current version
  ls -1 $VERSIONDIR/*.dat | while read filename
  do
    log Found version file $filename
    cp -f $filename $PATCHDIR/version 
    if [ $? -eq 0 ]
    then
      log Copied version file $filename to $PATCHDIR/version
    else
      log Could not copy version file $filename to $PATCHDIR/version
    fi
  done
}

stop_services()
{
  log Stopping services if needed
  mkdir $PATCHDIR/restart

  ps -ef | grep smbd | grep -v grep > /dev/null
  if [ $? -eq 0 ]
  then
    log Found smb service running, stopping for now
    # FIXING a previous bug where the link would be renamed to .org 
    /var/public/files/smb stop
    if [ $? -eq 0 ]
    then
       touch $PATCHDIR/restart/smb
       log Stopped smb service succesfully
    else
       log ERROR: Failed to stop smb service
    fi
  else
    log smb service not running,
  fi

  ps -ef | grep dhcpd | grep -v grep > /dev/null
  if [ $? -eq 0 ]
  then
    log Found dhcp service running, stopping for now
    /sbin/service dhcpd stop
    if [ $? -eq 0 ]
    then
       touch $PATCHDIR/restart/dhcpd
       log Stopped dhcpd service succesfully
    else
       log ERROR: Failed to stop dhcpd service
    fi
  else
    log dhcpd service not running,
  fi

  ps -ef | grep in.tftpd | grep -v grep > /dev/null
  if [ $? -eq 0 ]
  then
    log Found tftpd service running, stopping for now
    /sbin/service tftpd stop
    if [ $? -eq 0 ]
    then
       touch $PATCHDIR/restart/tftpd
       log Stopped tftpd service succesfully
    else
       log ERROR: Failed to stop tftpd service
    fi
  else
    log  tftpd service not running,
  fi

  ps -ef | grep syslog | grep -v grep > /dev/null
  if [ $? -eq 0 ]
  then
    log Found syslogd service running, stopping for now
    /sbin/service syslog stop
    if [ $? -eq 0 ]
    then
       touch $PATCHDIR/restart/syslog
       log Stopped syslogd service succesfully
    else
       log ERROR: Failed to stop syslog service
    fi
  else
    log  syslog service not running,
  fi

}

start_services()
{
  ls -1 $PATCHDIR/restart | while read service
  do
    log Starting service $service
    /sbin/service $service start
    if [ $? -eq 0 ]
    then
      log Started service $service succesfully
    else
      log ERROR: Could not start service $service 
    fi
  done
}

patch_current_winpe_files()
{
  log Updating Windows PE driver conf files without DRVLOAD variable and script
  ls -1 /var/public/conf/winpedrv | while read dir
  do
    DRIVERDIR=/var/public/conf/winpedrv/$dir
    if [ -d $DRIVERDIR ]
    then
      log Updating driver $dir
      DATFILE=/var/public/conf/winpedrv/$dir/driver.dat
      if [ -f $DATFILE ]
      then
        log Patching dat file $DATFILE
        sed -i 's/INFFILE/FILE1/g' $DATFILE
        sed -i 's/SYSFILE/FILE2/g' $DATFILE
      fi
      DRVLOAD=/var/public/conf/winpedrv/$dir/$dir.cmd
      if [ ! -f $DRVLOAD ]
      then
      
         log Creating cmd DRVLOAD file $DRVLOAD
         echo "drvload %1" > $DRVLOAD
         /usr/bin/unix2dos $DRVLOAD

         log Creating Adding DRVLOAD filename to $DATFILE
         /bin/grep "^DRVLOAD=" $DATFILE > /dev/null
         if [ $? -ne 0 ]
         then
           echo DRVLOAD=$dir.cmd >> $DATFILE
         fi
      fi
    fi
  done
}

patch_windows7_conf_files()
{
  log Updating windows 7 conf files with MOUNTTYPE_1=udf
  grep -l ^OS=windows7 /var/public/conf/os/*.dat | while read filename
  do
    log Updating $filename 
    grep ^MOUNTTYPE_1= $filename > /dev/null 2>&1
    if [ $? -ne 0 ]
    then
      log No current mounttype found, adding UDF type
      echo MOUNTTYPE_1=udf >> $filename
    else
      log Found current mounttype
    fi
  done
}

create_directories()
{
  log Creating Directories
  mkdir /var/public/www/ubuntu64
  mkdir /var/public/www/supportmatrix
  mkdir /var/public/www/supportmatrix/vmware
}


pre_copy_actions()
{
  log Pre Copy Actions
  create_directories
  stop_services
}


copy_files()
{
  log Copying Files
  copy_tftproot_files
  copy_bin_files
  copy_conf_files
  copy_cgi_bin_files
  copy_cgi_bin_os_files
  copy_www_files
  copy_www_support_files
  copy_www_support_vmware_files
  copy_template_files
  copy_js_files
}

check_patch20_fingerprint()
{
  log Running fingerprint check for version 20 patch

  PATCH20FINGERPRINTMD5=788cf4769bd4a1f6d0ed7af9b30bb357
  PATCH20FINGERPRINTFILE=/var/public/www/templates/esx5.tpl
  
  if [ -f $PATCH20FINGERPRINTFILE ]
  then
    log $PATCH20FINGERPRINTFILE found, now checking MD5
    currentmd5=`/usr/bin/md5sum $PATCH20FINGERPRINTFILE | awk '{print $1}'`
    if [ "$currentmd5" == "$PATCH20FINGERPRINTMD5" ]
    then
      log Correct MD5 sum found, adding version file for patch 20
      VERSIONFILE=$VERSIONDIR/uda20.dat
      echo VERSION=$VERSION\_Build20 > $VERSIONFILE
      log Wrote $VERSIONFILE
    else
      log Wrong MD5 sum found, not adding version file for patch 20
    fi
  else
    log $PATCH20FINGERPRINTFILE not found
  fi
}

post_copy_actions()
{
   log Post Copy Actions
   patch_windows7_conf_files
   patch_current_winpe_files
   overwrite_conf_files
   create_file_links
   change_file_permissions
   start_services
}

create_version_info()
{
  log Creating Versioninfo 
  VERSIONFILE=$VERSIONDIR/uda$BUILD.dat
  echo VERSION=$VERSION\_Build$BUILD > $VERSIONFILE
  log Wrote $VERSIONFILE
}

wrapup()
{
  WRAPUP=$PATCHTOPDIR/patch.$VERSION.$BUILD.$$.tgz
  log Wrapping up the results to $WRAPUP
  log Copying logfile to $PATCHDIR
  cp -f $LOGDIR/$LOGFILE $PATCHDIR
  tar -czf $WRAPUP $PATCHDIR
  rm -rf $PATCHDIR
  log For logfile see $LOGDIR/$LOGFILE
}

check_prereqs()
{
  log Checking Prereqs

  if [ ! -f $PATCHDIR/version/uda17.dat ]
  then
    log ERROR: Build 17 not found, exiting
    exit 1
  else
    log OK: Build 17 found
  fi

  if [ ! -f $PATCHDIR/version/uda19.dat ]
  then
    log ERROR: Build 19 not found, exiting
    exit 1
  else
    log OK: Build 19 found
  fi

  if [ ! -f $PATCHDIR/version/uda20.dat ]
  then
    log ERROR: Build 20 not found, exiting
    exit 1
  else
    log OK: Build 20 found
  fi

}

############################   Main loop ########################
prepare_upgrade
check_patch20_fingerprint
determine_current_versions
check_prereqs
pre_copy_actions
copy_files
post_copy_actions
create_version_info
wrapup
